#ifndef __SYSTEM_CONFIG_H_
#define __SYSTEM_CONFIG_H_


#define FPGA_DEVICE_FAMILY    "MachXO2"
#define PLATFORM_NAME         "platform_rev0"
#define USE_PLL               (0)
#define CPU_FREQUENCY         (25000000)


/* FOUND 1 CPU UNIT(S) */

/*
 * CPU Instance LM32 component configuration
 */
#define CPU_NAME "LM32"
#define CPU_EBA (0x00000000)
#define CPU_DIVIDE_ENABLED (1)
#define CPU_SIGN_EXTEND_ENABLED (1)
#define CPU_MULTIPLIER_ENABLED (1)
#define CPU_SHIFT_ENABLED (1)
#define CPU_DEBUG_ENABLED (1)
#define CPU_HW_BREAKPOINTS_ENABLED (0)
#define CPU_NUM_HW_BREAKPOINTS (0)
#define CPU_NUM_WATCHPOINTS (0)
#define CPU_ICACHE_ENABLED (0)
#define CPU_ICACHE_SETS (512)
#define CPU_ICACHE_ASSOC (1)
#define CPU_ICACHE_BYTES_PER_LINE (16)
#define CPU_DCACHE_ENABLED (0)
#define CPU_DCACHE_SETS (512)
#define CPU_DCACHE_ASSOC (1)
#define CPU_DCACHE_BYTES_PER_LINE (16)
#define CPU_DEBA (0x03000000)
#define CPU_CHARIO_IN        (1)
#define CPU_CHARIO_OUT       (1)
#define CPU_CHARIO_TYPE      "JTAG UART"

/*
 * uart component configuration
 */
#define UART_NAME  "uart"
#define UART_BASE_ADDRESS  (0x80000000)
#define UART_SIZE  (16)
#define UART_IRQ (0)
#define UART_CHARIO_IN        (1)
#define UART_CHARIO_OUT       (1)
#define UART_CHARIO_TYPE      "RS-232"
#define UART_ADDRESS_LOCK  (0)
#define UART_DISABLE  (0)
#define UART_MODEM  (0)
#define UART_WB_DAT_WIDTH  (8)
#define UART_WB_ADR_WIDTH  (4)
#define UART_BAUD_RATE  (115200)
#define UART_IB_SIZE  (4)
#define UART_OB_SIZE  (4)
#define UART_BLOCK_WRITE  (1)
#define UART_BLOCK_READ  (1)
#define UART_STDOUT_SIM  (0)
#define UART_STDOUT_SIMFAST  (0)
#define UART_RXRDY_ENABLE  (0)
#define UART_TXRDY_ENABLE  (0)
#define UART_INTERRUPT_DRIVEN  (1)
#define UART_LCR_DATA_BITS  (8)
#define UART_LCR_STOP_BITS  (1)
#define UART_LCR_PARITY_ENABLE  (0)
#define UART_LCR_PARITY_ODD  (0)
#define UART_LCR_PARITY_STICK  (0)
#define UART_LCR_SET_BREAK  (0)
#define UART_FIFO  (0)

/*
 * sdram component configuration
 */
#define SDRAM_NAME  "sdram"
#define SDRAM_BASE_ADDRESS  (0x02000000)
#define SDRAM_SIZE  (1048576)
#define SDRAM_IS_READABLE   (1)
#define SDRAM_IS_WRITABLE   (1)
#define SDRAM_LATTICE_FAMILY  "All"
#define SDRAM_LATTICE_DEVICE  "All"
#define SDRAM_SDRAM_FREQ  "100"
#define SDRAM_SDR_DATA_SIZE_32  (0)
#define SDRAM_SDR_DATA_SIZE_16  (1)
#define SDRAM_SDR_DATA_SIZE_8  (0)
#define SDRAM_SDR_DATA_SIZE_4  (0)
#define SDRAM_SDR_ROW_WIDTH  (12)
#define SDRAM_SDR_COL_WIDTH  (8)
#define SDRAM_SDR_BNK_WDTH  (2)
#define SDRAM_SDR_TMRD  (2)
#define SDRAM_SDR_TRP  (3)
#define SDRAM_SDR_TRFC  (7)
#define SDRAM_SDR_TRCD  (3)
#define SDRAM_SDR_TDAL  (5)
#define SDRAM_SDR_TREFI  (1564)
#define SDRAM_T100US  (10000)
#define SDRAM_ADDRESS_LOCK  (1)
#define SDRAM_DISABLE  (0)

/*
 * ebr component configuration
 */
#define EBR_NAME  "ebr"
#define EBR_BASE_ADDRESS  (0x01000000)
#define EBR_SIZE  (12288)
#define EBR_IS_READABLE   (1)
#define EBR_IS_WRITABLE   (1)
#define EBR_ADDRESS_LOCK  (1)
#define EBR_DISABLE  (0)
#define EBR_WB_DAT_WIDTH  (32)
#define EBR_INIT_FILE_NAME  "C:/work/project/artekit_demos/forza4_rev0/software/software/aaa.mem"
#define EBR_INIT_FILE_FORMAT  "hex"

/*
 * SPIFlash component configuration
 */
#define SPIFLASH_NAME  "SPIFlash"
#define SPIFLASH_BASE_ADDRESS  (0x00000000)
#define SPIFLASH_SIZE  (16777216)
#define SPIFLASH_IS_READABLE   (1)
#define SPIFLASH_IS_WRITABLE   (1)
#define SPIFLASH_LATTICE_FAMILY  "XP"
#define SPIFLASH_PORT_TYPE  "Memory"
#define SPIFLASH_WB_DAT_WIDTH  (32)
#define SPIFLASH_WB_ADR_WIDTH  (32)
#define SPIFLASH_PORT_TYPE  "Memory"
#define SPIFLASH_PORT_ENABLE  (0)
#define SPIFLASH_WB_DAT_WIDTH  (32)
#define SPIFLASH_WB_ADR_WIDTH  (32)
#define SPIFLASH_PORT_CONNECTION  "S;C;S:C"
#define SPIFLASH_CLOCK_SEL  (0)
#define SPIFLASH_PAGE_SIZE  (256)
#define SPIFLASH_SECTOR_SIZE  (32768)
#define SPIFLASH_PAGE_PRG_BUF_ENA  (0)
#define SPIFLASH_PAGE_READ_BUF_ENA  (0)
#define SPIFLASH_SPI_READ  (3)
#define SPIFLASH_SPI_FAST_READ  (11)
#define SPIFLASH_SPI_BYTE_PRG  (2)
#define SPIFLASH_SPI_PAGE_PRG  (2)
#define SPIFLASH_SPI_BLK1_ERS  (216)
#define SPIFLASH_SPI_BLK2_ERS  (216)
#define SPIFLASH_SPI_BLK3_ERS  (216)
#define SPIFLASH_SPI_CHIP_ERS  (199)
#define SPIFLASH_SPI_WRT_ENB  (6)
#define SPIFLASH_SPI_WRT_DISB  (4)
#define SPIFLASH_SPI_READ_STAT  (5)
#define SPIFLASH_SPI_WRT_STAT  (1)
#define SPIFLASH_SPI_PWD_DOWN  (185)
#define SPIFLASH_SPI_PWD_UP  (171)
#define SPIFLASH_SPI_DEV_ID  (159)
#define SPIFLASH_ADDRESS_LOCK  (1)
#define SPIFLASH_DISABLE  (0)


#endif /* __SYSTEM_CONFIG_H_ */
